#include "BSP/Leds/Leds.hpp"
#include <stdlib.h>

#define LED_COUNT 10
#define MAX_BRIGHTNESS 100

#define PROBABILITY(x) (rand() < (int)((double)RAND_MAX*(double)x/(double)100))
#define IS_BUTTON_PRESSED() (!(PINB & BIT4))

void InitPins();

volatile uint8 brightness[LED_COUNT];
int8  increment[LED_COUNT];
enum Mode { Stars, Constant, SlowStars };
Mode mode = Stars;


#pragma vector=TIMER0_COMPA_vect
__interrupt void pwm()
{
   PORTD |= BIT5;
   static uint8 counter = 0;
   static uint8 m_brightness[LED_COUNT];
   
   counter++;
   if (counter == MAX_BRIGHTNESS)
   {
      counter = 0;
      for (uint8 i=0; i<LED_COUNT; i++) m_brightness[i] = brightness[i];

      LED0_ON();
      LED1_ON();
      LED2_ON();
      LED3_ON();
      LED4_ON();
      LED5_ON();
      LED6_ON();
      LED7_ON();
      LED8_ON();
      LED9_ON();
   }
   
   if ( counter >= m_brightness[0] ) LED0_OFF();
   if ( counter >= m_brightness[1] ) LED1_OFF();   
   if ( counter >= m_brightness[2] ) LED2_OFF();
   if ( counter >= m_brightness[3] ) LED3_OFF();   
   if ( counter >= m_brightness[4] ) LED4_OFF();
   if ( counter >= m_brightness[5] ) LED5_OFF();   
   if ( counter >= m_brightness[6] ) LED6_OFF();
   if ( counter >= m_brightness[7] ) LED7_OFF();   
   if ( counter >= m_brightness[8] ) LED8_OFF();   
   if ( counter >= m_brightness[9] ) LED9_OFF();   
   
   PORTD &= ~BIT5;   
}


void Recount(uint8 led_id)
{
   brightness[led_id] += increment[led_id];
   if(brightness[led_id] >= MAX_BRIGHTNESS && increment[led_id] > 0) increment[led_id] = -1;
   if(brightness[led_id] == 0   && increment[led_id] < 0) increment[led_id] = 0;      
}

void ProcessButton()
{
   if (IS_BUTTON_PRESSED())
   {
      switch(mode)
      {
      case Stars:
         mode = Constant;
         break;
         
      case Constant:
         mode = SlowStars;
         break;
         
      case SlowStars:
         mode = Stars;
         break;
      }
      
      while(IS_BUTTON_PRESSED()) {;}
   }
}


void main()
{
   InitPins();
   TCCR0A = (1<<WGM01);
   TCCR0B = (1<<CS00);
   TIMSK |= (1<<OCIE0A);
   OCR0A = 0x7f;
   EINT();
      
   while(1)
   {
      switch(mode)
      {
         
      case Stars:
         
         __delay_cycles(4000);
         
         
         for (uint8 i=0; i<LED_COUNT; i++) 
         {
            Recount(i);
            if (PROBABILITY(1) && increment[i] == 0) increment[i] = 1;
         }
         break;

      case Constant:
         
         __delay_cycles(4000);
         
         
         for (uint8 i=0; i<LED_COUNT; i++) 
         {
            brightness[i] = MAX_BRIGHTNESS;
         }
         break;


      case SlowStars:
         
         __delay_cycles(20000);
         
         
         for (uint8 i=0; i<LED_COUNT; i++) 
         {
            Recount(i);
            if (PROBABILITY(4) && increment[i] <= 0) increment[i] = 1;
         }
         break;
      }
      
      ProcessButton();
   }
}


void InitPins()
{
   DDRB = BIT0 | BIT1 | BIT2 | BIT3;
   PORTB = BIT4;
   DDRD = 0xff;
}
